@@ -57,9 +57,89 @@
 /*
 /*	The text below provides only a parameter summary. See
 /*	\fBpostconf\fR(5) for more details including examples.
-/* STARTTLS SUPPORT CONTROLS
+/* STARTTLS GLOBAL CONTROLS
 /* .ad
 /* .fi
+/*	The following settings are global and therefore cannot be
+/*	overruled by information specified in a \fBtlsproxy\fR(8)
+/*	client request.
+/* .IP "\fBtls_append_default_CA (no)\fR"
+/*	Append the system-supplied default Certification Authority
+/*	certificates to the ones specified with *_tls_CApath or *_tls_CAfile.
+/* .IP "\fBtls_daemon_random_bytes (32)\fR"
+/*	The number of pseudo-random bytes that an \fBsmtp\fR(8) or \fBsmtpd\fR(8)
+/*	process requests from the \fBtlsmgr\fR(8) server in order to seed its
+/*	internal pseudo random number generator (PRNG).
+/* .IP "\fBtls_high_cipherlist (see 'postconf -d' output)\fR"
+/*	The OpenSSL cipherlist for "high" grade ciphers.
+/* .IP "\fBtls_medium_cipherlist (see 'postconf -d' output)\fR"
+/*	The OpenSSL cipherlist for "medium" or higher grade ciphers.
+/* .IP "\fBtls_low_cipherlist (see 'postconf -d' output)\fR"
+/*	The OpenSSL cipherlist for "low" or higher grade ciphers.
+/* .IP "\fBtls_export_cipherlist (see 'postconf -d' output)\fR"
+/*	The OpenSSL cipherlist for "export" or higher grade ciphers.
+/* .IP "\fBtls_null_cipherlist (eNULL:!aNULL)\fR"
+/*	The OpenSSL cipherlist for "NULL" grade ciphers that provide
+/*	authentication without encryption.
+/* .IP "\fBtls_eecdh_strong_curve (prime256v1)\fR"
+/*	The elliptic curve used by the Postfix SMTP server for sensibly
+/*	strong
+/*	ephemeral ECDH key exchange.
+/* .IP "\fBtls_eecdh_ultra_curve (secp384r1)\fR"
+/*	The elliptic curve used by the Postfix SMTP server for maximally
+/*	strong
+/*	ephemeral ECDH key exchange.
+/* .IP "\fBtls_disable_workarounds (see 'postconf -d' output)\fR"
+/*	List or bit-mask of OpenSSL bug work-arounds to disable.
+/* .IP "\fBtls_preempt_cipherlist (no)\fR"
+/*	With SSLv3 and later, use the Postfix SMTP server's cipher
+/*	preference order instead of the remote client's cipher preference
+/*	order.
+/* .PP
+/*	Available in Postfix version 2.9 and later:
+/* .IP "\fBtls_legacy_public_key_fingerprints (no)\fR"
+/*	A temporary migration aid for sites that use certificate
+/*	\fIpublic-key\fR fingerprints with Postfix 2.9.0..2.9.5, which use
+/*	an incorrect algorithm.
+/* .PP
+/*	Available in Postfix version 2.11-3.1:
+/* .IP "\fBtls_dane_digest_agility (on)\fR"
+/*	Configure RFC7671 DANE TLSA digest algorithm agility.
+/* .IP "\fBtls_dane_trust_anchor_digest_enable (yes)\fR"
+/*	Enable support for RFC 6698 (DANE TLSA) DNS records that contain
+/*	digests of trust-anchors with certificate usage "2".
+/* .PP
+/*	Available in Postfix version 2.11 and later:
+/* .IP "\fBtlsmgr_service_name (tlsmgr)\fR"
+/*	The name of the \fBtlsmgr\fR(8) service entry in master.cf.
+/* .PP
+/*	Available in Postfix version 3.0 and later:
+/* .IP "\fBtls_session_ticket_cipher (Postfix >= 3.0: aes-256-cbc, Postfix < 3.0: aes-128-cbc)\fR"
+/*	Algorithm used to encrypt RFC5077 TLS session tickets.
+/* .IP "\fBopenssl_path (openssl)\fR"
+/*	The location of the OpenSSL command line program \fBopenssl\fR(1).
+/* .PP
+/*	Available in Postfix version 3.2 and later:
+/* .IP "\fBtls_eecdh_auto_curves (see 'postconf -d' output)\fR"
+/*	The prioritized list of elliptic curves supported by the Postfix
+/*	SMTP client and server.
+/* .PP
+/*	Available in Postfix version 3.4 and later:
+/* .IP "\fBtls_server_sni_maps (empty)\fR"
+/*	Optional lookup tables that map names received from remote SMTP
+/*	clients via the TLS Server Name Indication (SNI) extension to the
+/*	appropriate keys and certificate chains.
+/* STARTTLS SERVER CONTROLS
+/* .ad
+/* .fi
+/*	These settings are clones of Postfix SMTP server settings.
+/*	They allow \fBtlsproxy\fR(8) to load the same certificate
+/*	and private key information as the Postfix SMTP server,
+/*	before dropping privileges, so that the key files can be
+/*	kept read-only for root. These settings can currently not
+/*	be overruled by information in a \fBtlsproxy\fR(8) client
+/*	request, but that limitation may be removed in a future
+/*	version.
 /* .IP "\fBtlsproxy_tls_CAfile ($smtpd_tls_CAfile)\fR"
 /*	A file containing (PEM format) CA certificates of root CAs
 /*	trusted to sign either remote SMTP client certificates or intermediate
@@ -134,27 +214,18 @@
 /*	The SMTP TLS security level for the Postfix \fBtlsproxy\fR(8) server;
 /*	when a non-empty value is specified, this overrides the obsolete
 /*	parameters smtpd_use_tls and smtpd_enforce_tls.
-/* .PP
-/*	Available in Postfix version 2.11 and later:
-/* .IP "\fBtlsmgr_service_name (tlsmgr)\fR"
-/*	The name of the \fBtlsmgr\fR(8) service entry in master.cf.
-/* .PP
-/*	Available in Postfix version 3.4 and later:
 /* .IP "\fBtlsproxy_tls_chain_files ($smtpd_tls_chain_files)\fR"
 /*	Files with the Postfix \fBtlsproxy\fR(8) server keys and certificate
 /*	chains in PEM format.
-/* .IP "\fBtls_server_sni_maps (empty)\fR"
-/*	Optional lookup tables that map names received from remote SMTP
-/*	clients via the TLS Server Name Indication (SNI) extension to the
-/*	appropriate keys and certificate chains.
-/* TLS CLIENT CONTROLS
+/* STARTTLS CLIENT CONTROLS
 /* .ad
 /* .fi
-/*	These parameters are clones of SMTP client settings. They
-/*	allow \fBtlsproxy\fR(8) to load the same certificate and
-/*	private key information as the SMTP client, before dropping
-/*	privileges, so that the key files can be kept read-only for
-/*	root.
+/*	These settings are clones of Postfix SMTP client settings.
+/*	They allow \fBtlsproxy\fR(8) to load the same certificate
+/*	and private key information as the Postfix SMTP client,
+/*	before dropping privileges, so that the key files can be
+/*	kept read-only for root. Some settings may be overruled by
+/*	information in a \fBtlsproxy\fR(8) client request.
 /* .PP
 /*	Available in Postfix version 3.4 and later:
 /* .IP "\fBtlsproxy_client_CAfile ($smtp_tls_CAfile)\fR"
@@ -296,6 +367,7 @@
  /*
   * Global library.
   */
+#include <been_here.h>
 #include <mail_proto.h>
 #include <mail_params.h>
 #include <mail_conf.h>
@@ -426,16 +498,16 @@ char   *var_tlsp_clnt_policy;
   * TLS per-process status.
   */
 static TLS_APPL_STATE *tlsp_server_ctx;
-static TLS_APPL_STATE *tlsp_client_ctx;
 static bool tlsp_pre_jail_done;
 static int ask_client_cert;
+static char *tlsp_pre_jail_client_param_key;	/* pre-jail global params */
+static char *tlsp_pre_jail_client_init_key;	/* pre-jail init props */
 
  /*
   * TLS per-client status.
   */
-static HTABLE *tlsp_client_app_cache;
-static char *tlsp_pre_jail_tls_param_key;
-static char *tlsp_pre_jail_client_props_key;
+static HTABLE *tlsp_client_app_cache;	/* per-client init props */
+static BH_TABLE *tlsp_params_mismatch_filter;	/* per-client nag filter */
 
  /*
   * Error handling: if a function detects an error, then that function is
@@ -874,7 +946,13 @@ static int tlsp_client_start_pre_handshake(TLSP_STATE *state)
 {
     state->client_start_props->ctx = state->appl_state;
     state->client_start_props->fd = state->ciphertext_fd;
-    state->tls_context = tls_client_start(state->client_start_props);
+    /* These predicates and warning belong inside tls_client_start(). */
+    if (!TLS_DANE_BASED(state->client_start_props->tls_level)
+	|| tls_dane_avail())
+	state->tls_context = tls_client_start(state->client_start_props);
+    else
+	msg_warn("%s: DANE requested, but not available",
+		 state->client_start_props->namaddr);
     if (state->tls_context != 0)
 	return (TLSP_STAT_OK);
 
@@ -1051,7 +1129,8 @@ static void tlsp_log_config_diff(const char *server_cfg, const char *client_cfg)
 	if (strcmp(server_field, client_field) != 0) {
 	    if (LEN(diff_summary) > 0)
 		vstring_sprintf_append(diff_summary, "; ");
-	    vstring_sprintf_append(diff_summary, "server '%s' != client '%s'",
+	    vstring_sprintf_append(diff_summary,
+				   "(server) '%s' != (client) '%s'",
 				   server_field, client_field);
 	}
     }
@@ -1062,70 +1141,95 @@ static void tlsp_log_config_diff(const char *server_cfg, const char *client_cfg)
     myfree(saved_server);
 }
 
- /*
-  * Macro for readability.
-  */
-#define TLSP_CLIENT_INIT(params, props, a1, a2, a3, a4, a5, a6, a7, a8, a9, \
-    a10, a11, a12, a13, a14) \
-    tlsp_client_init(params, TLS_CLIENT_INIT_ARGS((props), a1, a2, a3, a4, \
-    a5, a6, a7, a8, a9, a10, a11, a12, a13, a14))
-
 /* tlsp_client_init - initialize a TLS client engine */
 
-static TLS_APPL_STATE *tlsp_client_init(TLS_PARAMS *tls_params,
-				          TLS_CLIENT_INIT_PROPS *init_props)
+static TLS_APPL_STATE *tlsp_client_init(TLS_CLIENT_PARAMS *tls_params,
+				          TLS_CLIENT_INIT_PROPS *init_props,
+					        int dane_based)
 {
     TLS_APPL_STATE *appl_state;
     VSTRING *param_buf;
     char   *param_key;
     VSTRING *init_buf;
     char   *init_key;
+    VSTRING *init_buf_for_hashing;
+    char   *init_key_for_hashing;
+    int     log_hints = 0;
 
     /*
      * Use one TLS_APPL_STATE object for all requests that specify the same
      * TLS_CLIENT_INIT_PROPS. Each TLS_APPL_STATE owns an SSL_CTX, which is
-     * expensive to create.
+     * expensive to create. Bug: TLS_CLIENT_PARAMS are not used when creating
+     * a TLS_APPL_STATE instance.
      * 
      * First, compute the TLS_APPL_STATE cache lookup key. Save a copy of the
-     * key that corresponds to the pre-jail internal request, which uses the
-     * tlsproxy_client_* settings.
+     * pre-jail request TLS_CLIENT_PARAMS and TLSPROXY_CLIENT_INIT_PROPS
+     * settings, so that we can detect post-jail requests that do not match.
+     * 
+     * Workaround: salt the hash-table key with DANE on/off info. This avoids
+     * cross-talk between DANE and non-DANE sessions. Postfix DANE support
+     * modifies SSL_CTX to override certificate verification because there is
+     * no other way to do this before OpenSSL 1.1.0.
      */
     param_buf = vstring_alloc(100);
-    param_key = tls_proxy_params_to_string(param_buf, tls_params);
+    param_key = tls_proxy_client_param_with_names_to_string(
+						     param_buf, tls_params);
     init_buf = vstring_alloc(100);
-    init_key = tls_proxy_client_init_to_string(init_buf, init_props);
+    init_key = tls_proxy_client_init_with_names_to_string(
+						      init_buf, init_props);
+    init_buf_for_hashing = vstring_alloc(100);
+    init_key_for_hashing = STR(vstring_sprintf(init_buf_for_hashing, "%s%d\n",
+					       init_key, dane_based));
     if (tlsp_pre_jail_done == 0) {
-	if (tlsp_pre_jail_tls_param_key != 0
-	    || tlsp_pre_jail_client_props_key != 0)
-	    msg_panic("tlsp_client_init: multiple pre-jail calls");
-	tlsp_pre_jail_tls_param_key = mystrdup(param_key);
-	tlsp_pre_jail_client_props_key = mystrdup(init_key);
+	if (tlsp_pre_jail_client_param_key == 0
+	    || tlsp_pre_jail_client_init_key == 0) {
+	    tlsp_pre_jail_client_param_key = mystrdup(param_key);
+	    tlsp_pre_jail_client_init_key = mystrdup(init_key);
+	} else if (strcmp(tlsp_pre_jail_client_param_key, param_key) != 0
+		   || strcmp(tlsp_pre_jail_client_init_key, init_key) != 0) {
+	    msg_panic("tlsp_client_init: too many pre-jail calls");
+	}
     }
 
     /*
-     * Log a warning if a post-jail request differs from the tlsproxy_client_*
-     * or tls_mumble settings AND the request specifies file/directory
-     * pathname arguments. Pathname differences are problematic after chroot
-     * (pathname resolution) and after dropping privileges (key files must be
-     * root read-only).
-     * 
-     * We can eliminate this complication by adding code that opens a cert/key
-     * lookup table at pre-jail time, and by reading cert/key info on-the-fly
-     * from that table.
+     * Log a warning if a post-jail request uses unexpected TLS_CLIENT_PARAMS
+     * settings. Bug: TLS_CLIENT_PARAMS settings are not used when creating a
+     * TLS_APPL_STATE instance; this makes a mismatch of TLS_CLIENT_PARAMS
+     * settings problematic.
      */
+    if (tlsp_pre_jail_done
+	&& !been_here_fixed(tlsp_params_mismatch_filter, param_key)
+	&& strcmp(tlsp_pre_jail_client_param_key, param_key) != 0) {
+	msg_warn("request from tlsproxy client with unexpected settings");
+	tlsp_log_config_diff(tlsp_pre_jail_client_param_key, param_key);
+	log_hints = 1;
+    }
+
+    /*
+     * Look up the cached TLS_APPL_STATE for this tls_client_init request.
+     */
+    if ((appl_state = (TLS_APPL_STATE *)
+	 htable_find(tlsp_client_app_cache, init_key_for_hashing)) == 0) {
+
+	/*
+	 * Before creating a TLS_APPL_STATE instance, log a warning if a
+	 * post-jail request differs from the saved pre-jail request AND the
+	 * post-jail request specifies file/directory pathname arguments.
+	 * Unexpected requests containing pathnames are problematic after
+	 * chroot (pathname resolution) and after dropping privileges (key
+	 * files must be root read-only). Unexpected requests are not a
+	 * problem as long as they contain no pathnames (for example a
+	 * tls_loglevel change).
+	 * 
+	 * We could eliminate some of this complication by adding code that
+	 * opens a cert/key lookup table at pre-jail time, and by reading
+	 * cert/key info on-the-fly from that table. But then all requests
+	 * would still have to specify the same table.
+	 */
 #define NOT_EMPTY(x) ((x) && *(x))
 
-    else {
-	int     log_hints = 0;
-
-	if (tlsp_pre_jail_tls_param_key != 0
-	    && strcmp(tlsp_pre_jail_tls_param_key, param_key) != 0) {
-	    msg_warn("request from Postfix client with unexpected settings");
-	    tlsp_log_config_diff(tlsp_pre_jail_tls_param_key, param_key);
-	    log_hints = 1;
-	}
-	if (tlsp_pre_jail_client_props_key != 0
-	    && strcmp(tlsp_pre_jail_client_props_key, init_key) != 0
+	if (tlsp_pre_jail_done
+	    && strcmp(tlsp_pre_jail_client_init_key, init_key) != 0
 	    && (NOT_EMPTY(init_props->chain_files)
 		|| NOT_EMPTY(init_props->cert_file)
 		|| NOT_EMPTY(init_props->key_file)
@@ -1136,31 +1240,28 @@ static TLS_APPL_STATE *tlsp_client_init(TLS_PARAMS *tls_params,
 		|| NOT_EMPTY(init_props->CAfile)
 		|| NOT_EMPTY(init_props->CApath))) {
 	    msg_warn("request from tlsproxy client with unexpected settings");
-	    tlsp_log_config_diff(tlsp_pre_jail_client_props_key, init_key);
+	    tlsp_log_config_diff(tlsp_pre_jail_client_init_key, init_key);
 	    log_hints = 1;
 	}
-	if (log_hints)
-	    msg_warn("to avoid this warning, 1) identify the tlsproxy "
-		     "client that is making this request, 2) configure "
-		     "a custom tlsproxy service with settings that "
-		     "match that tlsproxy client, and 3) configure "
-		     "that tlsproxy client with a tlsproxy_service_name "
-		     "setting that resolves to that custom tlsproxy "
-		     "service");
     }
+    if (log_hints)
+	msg_warn("to avoid this warning, 1) identify the tlsproxy "
+		 "client that is making this request, 2) configure "
+		 "a custom tlsproxy service with settings that "
+		 "match that tlsproxy client, and 3) configure "
+		 "that tlsproxy client with a tlsproxy_service_name "
+		 "setting that resolves to that custom tlsproxy "
+		 "service");
 
     /*
-     * Now, back to our regular program: look up the cached TLS_APPL_STATE
-     * for this tls_client_init request, or create one and add it the
-     * TLS_APPL_STATE cache. TLS_APPL_STATE creation may fail when a
-     * post-jail request specifies unexpected cert/key information, but that
-     * is OK because we already logged a warning with configuration
-     * suggestions.
+     * TLS_APPL_STATE creation may fail when a post-jail request specifies
+     * unexpected cert/key information, but that is OK because we already
+     * logged a warning with configuration suggestions.
      */
-    if ((appl_state = (TLS_APPL_STATE *)
-	 htable_find(tlsp_client_app_cache, init_key)) == 0
+    if (appl_state == 0
 	&& (appl_state = tls_client_init(init_props)) != 0) {
-	(void) htable_enter(tlsp_client_app_cache, init_key, (void *) appl_state);
+	(void) htable_enter(tlsp_client_app_cache, init_key_for_hashing,
+			    (void *) appl_state);
 
 	/*
 	 * To maintain sanity, allow partial SSL_write() operations, and
@@ -1169,11 +1270,11 @@ static TLS_APPL_STATE *tlsp_client_init(TLS_PARAMS *tls_params,
 	 * a mailing list, but is not supported by documentation. If this
 	 * code stops working then no-one can be held responsible.
 	 */
-	if (appl_state)
-	    SSL_CTX_set_mode(appl_state->ssl_ctx,
-			     SSL_MODE_ENABLE_PARTIAL_WRITE
-			     | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
+	SSL_CTX_set_mode(appl_state->ssl_ctx,
+			 SSL_MODE_ENABLE_PARTIAL_WRITE
+			 | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
     }
+    vstring_free(init_buf_for_hashing);
     vstring_free(init_buf);
     vstring_free(param_buf);
     return (appl_state);
@@ -1263,7 +1364,7 @@ static void tlsp_get_request_event(int event, void *context)
     case TLS_PROXY_FLAG_ROLE_CLIENT:
 	state->is_server_role = 0;
 	if (attr_scan(plaintext_stream, ATTR_FLAG_STRICT,
-		      RECV_ATTR_FUNC(tls_proxy_params_scan,
+		      RECV_ATTR_FUNC(tls_proxy_client_param_scan,
 				     (void *) &state->tls_params),
 		      RECV_ATTR_FUNC(tls_proxy_client_init_scan,
 				     (void *) &state->client_init_props),
@@ -1275,7 +1376,8 @@ static void tlsp_get_request_event(int event, void *context)
 	    return;
 	}
 	state->appl_state = tlsp_client_init(state->tls_params,
-					     state->client_init_props);
+					     state->client_init_props,
+		      TLS_DANE_BASED(state->client_start_props->tls_level));
 	ready = state->appl_state != 0;
 	break;
     case TLS_PROXY_FLAG_ROLE_SERVER:
@@ -1554,8 +1656,9 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
      * Initialize the TLS data before entering the chroot jail.
      */
     if (clnt_use_tls || var_tlsp_clnt_per_site[0] || var_tlsp_clnt_policy[0]) {
-	TLS_PARAMS tls_params;
+	TLS_CLIENT_PARAMS tls_params;
 	TLS_CLIENT_INIT_PROPS init_props;
+	int     dane_based_mode;
 
 	tls_pre_jail_init(TLS_ROLE_CLIENT);
 
@@ -1566,25 +1669,27 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
 	 * Large parameter lists are error-prone, so we emulate a language
 	 * feature that C does not have natively: named parameter lists.
 	 */
-	tlsp_client_ctx =
-	    TLSP_CLIENT_INIT(tls_proxy_params_from_config(&tls_params),
-			     &init_props,
-			     log_param = var_tlsp_clnt_logparam,
-			     log_level = var_tlsp_clnt_loglevel,
-			     verifydepth = var_tlsp_clnt_scert_vd,
-			     cache_type = TLS_MGR_SCACHE_SMTP,
-			     chain_files = var_tlsp_clnt_chain_files,
-			     cert_file = var_tlsp_clnt_cert_file,
-			     key_file = var_tlsp_clnt_key_file,
-			     dcert_file = var_tlsp_clnt_dcert_file,
-			     dkey_file = var_tlsp_clnt_dkey_file,
-			     eccert_file = var_tlsp_clnt_eccert_file,
-			     eckey_file = var_tlsp_clnt_eckey_file,
-			     CAfile = var_tlsp_clnt_CAfile,
-			     CApath = var_tlsp_clnt_CApath,
-			     mdalg = var_tlsp_clnt_fpt_dgst);
-	if (tlsp_client_ctx == 0)
-	    msg_warn("TLS client initialization failed");
+	(void) tls_proxy_client_param_from_config(&tls_params);
+	(void) TLS_CLIENT_INIT_ARGS(&init_props,
+				    log_param = var_tlsp_clnt_logparam,
+				    log_level = var_tlsp_clnt_loglevel,
+				    verifydepth = var_tlsp_clnt_scert_vd,
+				    cache_type = TLS_MGR_SCACHE_SMTP,
+				    chain_files = var_tlsp_clnt_chain_files,
+				    cert_file = var_tlsp_clnt_cert_file,
+				    key_file = var_tlsp_clnt_key_file,
+				    dcert_file = var_tlsp_clnt_dcert_file,
+				    dkey_file = var_tlsp_clnt_dkey_file,
+				    eccert_file = var_tlsp_clnt_eccert_file,
+				    eckey_file = var_tlsp_clnt_eckey_file,
+				    CAfile = var_tlsp_clnt_CAfile,
+				    CApath = var_tlsp_clnt_CApath,
+				    mdalg = var_tlsp_clnt_fpt_dgst);
+	for (dane_based_mode = 0; dane_based_mode < 2; dane_based_mode++) {
+	    if (tlsp_client_init(&tls_params, &init_props,
+				 dane_based_mode) == 0)
+		msg_warn("TLS client initialization failed");
+	}
     }
 
     /*
@@ -1592,6 +1697,12 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
      * post-jail.
      */
     tlsp_pre_jail_done = 1;
+
+    /*
+     * Bug: TLS_CLIENT_PARAMS attributes are not used when creating a
+     * TLS_APPL_STATE instance; we can only warn about attribute mismatches.
+     */
+    tlsp_params_mismatch_filter = been_here_init(BH_BOUND_NONE, BH_FLAG_NONE);
 }
 
 MAIL_VERSION_STAMP_DECLARE;
